--[[
Plugin for showing a warning text in the middle of the screen (or desired place)
]]

----------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("WarningFrame")
if not plugin then return end

----------------------------------
-- Locals
local LSM = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0-Plugins")

local Party
local db

----------------------------------
-- Defaults

local defaults = {
	profile = {
		Width = 180,
		DefaultColor = {0, 0.5, 1},
		DefaultSize = 40,
		DefaultSound = "RW: Tata",
		PlaySound = true,
		UseFlash = true,
		PluginEnabled = true,
	}
}

-- Pupulate default settings
for i, setting in pairs(RW.defaults.profile) do
	defaults.profile[i] = setting
end

----------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	self.db.RegisterCallback(self, "OnProfileChanged", "Reset")
	self.db.RegisterCallback(self, "OnProfileCopied", "Reset")
	self.db.RegisterCallback(self, "OnProfileReset", "Reset")
end

function plugin:PluginEnable()
	if not db.PluginEnabled then
		self:Disable()
		return
	end

	self:RegisterMessage("RW_ConfigMode")
	self:RegisterMessage("RW_DefaultsChanged")
	self:RegisterMessage("RW_Warning")
	Party = RW:GetPlugin("Party")
end

function plugin:PluginDisable()
end

-----------------------------------
-- Anchor

local function dragStart(self)
	self:StartMoving()
end
local function dragStop(self)
	self:StopMovingOrSizing()
	local s = self:GetEffectiveScale()
	db.x = self:GetLeft() * s
	db.y = self:GetBottom() * s
end

local function getAnchor()
	if plugin.anchor then return plugin.anchor end
	local frame = RW:CreateAnchor(plugin:GetName(), UIParent, "Warning frame")
	frame:SetScript("OnDragStart", dragStart)
	frame:SetScript("OnDragStop", dragStop)
	frame:ClearAllPoints()
	local s = frame:GetEffectiveScale()
	if db.x then
		frame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
	else
		frame:SetPoint("CENTER", UIParent, "CENTER", -0, 70)
	end
	frame:SetWidth(db.Width)
	plugin.anchor = frame
	return frame
end

local function getWarningFrame()
	if plugin.warningFrame then return plugin.warningFrame end
	local db = plugin.db.profile
	local anchor = getAnchor()
	
	local frame = CreateFrame("Frame", "RW_WarningFrame")
	frame:SetPoint("BOTTOM", anchor, "TOP")
	frame:SetWidth(db.Width)
	frame:SetHeight(1)
	
	local text = frame:CreateFontString(frame:GetName().."String", "ARTWORK")
	text:SetPoint("BOTTOM", frame, "TOP")
	
	local icon = frame:CreateTexture(frame:GetName().."Icon", "ARTWORK")
	icon:SetPoint("RIGHT", text, "LEFT", -10, 0)
	
	frame:SetScript("OnUpdate", function(self, elapsed)
		self.timer = self.timer - elapsed
		if self.timer >= 3 and self.timer <= 4 then
			LowHealthFrame:SetAlpha(self.timer - 3)
		elseif self.timer <= 0.1 then
			frame:Hide()
		elseif self.timer <= 2 then
			frame:SetAlpha(self.timer/2)
		end
	end)
	
	frame.text = text
	frame.icon = icon
	plugin.warningFrame = frame
	plugin:ApplySettings()
	return frame
end

-----------------------------------
-- Warning functions

-- Reset position
function plugin:Reset()
	db = self.db.profile
	if self.anchor then
		self.anchor:ClearAllPoints()
		local s = self.anchor:GetEffectiveScale()
		if db.x then
			self.anchor:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", db.x / s, db.y / s)
		else
			self.anchor:SetPoint("CENTER", UIParent, "CENTER", -0, 70)
		end
		
		plugin:ApplySettings()
	end
end

-- Apply the visual settings
function plugin:ApplySettings()
	if not plugin.warningFrame then return end
	local frame = getWarningFrame()
	
	frame.text:SetTextHeight(1)
	frame.text:SetFont(LSM:Fetch("font", db.Font), db.DefaultSize, "OUTLINE")
	frame.text:SetText(frame.text:GetText() or "T")
	frame.text:SetTextColor(unpack(db.DefaultColor))
	frame.icon:SetWidth(frame.text:GetStringHeight())
	frame.icon:SetHeight(frame.text:GetStringHeight())
end

--- Show a warning message
-- @param text The text to shows 
-- @param icon Optional icon to the left of the text
-- @param color Optional color for the warning, else it will be the default color
function plugin:RW_Warning(message, text, icon, color, sound, flash)
	local frame = getWarningFrame()
	local c = color or self.db.profile.DefaultColor
	frame.text:SetText(text)
	frame.text:SetTextColor(unpack(c))
	frame.icon:SetTexture(icon)
	frame:Show()
	frame:SetAlpha(1)
	frame.timer = 5
	
	if db.PlaySound then
		PlaySoundFile(LSM:Fetch("sound", sound or db.DefaultSound))
	end
	if db.UseFlash and flash then
		self:SendMessage("RW_Flash", color)
	end
end

function plugin:RW_ConfigMode(message, start)
	if start then
		getAnchor():Show()
		getWarningFrame():Show()
		getWarningFrame():SetAlpha(1)
		getWarningFrame().timer = 11111111
		getWarningFrame().text:SetText("Warning Frame")
	else
		getAnchor():Hide()
		getWarningFrame():Hide()
		getWarningFrame().timer = 0
	end
end

function plugin:RW_DefaultsChanged(message, option, value)
	if 	self.db.profile[option] and option == "Font" then
		self.db.profile[option] = value
		if self.anchor then
			self:ApplySettings()
		end
	end
end

function plugin:GetOptions()
	self.category = "Alert Plugins"
	
	local icon = [[Interface\Icons\INV_Misc_ClothScrap_02]]
	
	local option = {
		type = "group",
		name = "Warnings",
		icon = icon,
		set = function(info, value) 
			db[info[#info]] = value
			plugin:ApplySettings()
		end,
		get = function(info) return db[info[#info]] end,
		args = {
			image = {
				type = "description",
				name = L.WARNING_FRAME_DESC,
				image = icon,
				width = "full",
				imageWidth = 50,
				imageHeight = 50,
				imageCoords = {0.1,0.9,0.1,0.9},
				order = 1,
			},
			enabled = {
				type = "toggle",
				name = "Enabled",
				set = function(info, value) 
					db.PluginEnabled = value
					if not db.PluginEnabled then
						self:Disable()
					else
						self:Enable()
					end
				end,
				get = function(info) return db.PluginEnabled end,
				order = 2,
			},
			headingBreak = {
				type = "header",
				name = "",
				order = 3,
			},
			UseFlash = {
				type = "toggle",
				name = "Screen flash",
				desc = "For flash settings see separate plugin settings (Screen Flash)",
				order = 21,
			},			
			PlaySound = {
				type = "toggle",
				name = "Play sound",
				order = 22,
			},
			DefaultSound = {
				type = "select",
				name = "Default sound",
				dialogControl = "LSM30_Sound",
				values = AceGUIWidgetLSMlists.sound,
				order = 23,
			},
			newLN1 = {
				type = "description",
				name = "",
				order = 30,
			},
			DefaultColor = {
				type = "color",
				name = "Default color",
				desc = "This is the default color used, will be overridden by any warning that has a special color set",
				get = function() return unpack(db.DefaultColor) end,
				set = function(info, r,g,b,a)
					local c = db.DefaultColor
					c[1], c[2], c[3] = r, g, b
					plugin:ApplySettings()
				end,
				order = 31,
			},
			DefaultSize = {
				type = "range",
				name = "Font size",
				min = 5,
				max = 100,
				step = 1,
				order = 32,
			},
			newLN10 = {
				type = "description",
				name = "",
				order = 90,
			},
			Test = {
				type = "execute",
				name = "Test",
				func = function() 
					local spellName, rank, icon = RW:GetRandomSpell()
					plugin:SendMessage("RW_Warning", spellName ..(rank ~= "" and (" ("..rank..")" ) or "" ), icon, nil, nil, true) 
				end,
				order = 99,
			},
			Reset = {
				type = "execute",
				name = L.RESET_SETTINGS,
				desc = L.RESET_SETTINGS_DESC,
				confirm = true,
				func = function() self.db:ResetProfile() end,
				order = 100,
			},
		}
	}
	
	return option
end
